<?php

namespace WcPsigate\Helpers;

use WC_PsiGate;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Description
 *
 * @since  1.7.0
 * @author VanboDevelops | Ivan Andreev
 *
 *        Copyright: (c) 2018 VanboDevelops
 *        License: GNU General Public License v3.0
 *        License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */
class Page_Helpers {
	
	/**
	 * Is the customer on the update payment method page
	 * @since 1.7.0
	 * @return bool
	 */
	public static function is_update_payment_method_page() {
		global $wp;
		
		$page_id = wc_get_page_id( 'myaccount' );
		
		return $page_id && is_page( $page_id ) && isset( $wp->query_vars['update-payment-method'] );
	}
	
	/**
	 * Is the customer on the payment methods page
	 * @since 1.7.0
	 * @return bool
	 */
	public static function is_payment_methods_page() {
		global $wp;
		
		$page_id = wc_get_page_id( 'myaccount' );
		
		return $page_id && is_page( $page_id ) && isset( $wp->query_vars['payment-methods'] );
	}
	
	/**
	 * Is this the change method page
	 * @since 1.7.0
	 * @return bool
	 */
	public static function is_change_method_page() {
		
		if ( ! class_exists( 'WC_Subscriptions_Change_Payment_Gateway' ) ) {
			return false;
		}
		
		if ( ! isset( $_GET['pay_for_order'] ) ) {
			return false;
		}
		return \WC_Subscriptions_Change_Payment_Gateway::$is_request_to_change_payment;
	}
	
	/**
	 * Is this the pay for order page.
	 * Definition:  The page the customers go to when they want to pay for an existing order.
	 *              Usually, goes through "My Account > Orders > Pay button"
	 *
	 * @since 1.7.0
	 * @return bool
	 */
	public static function is_pay_for_order_page() {
		if ( is_checkout_pay_page() && 'true' === Retrievers::get_field( 'pay_for_order', $_GET, '' ) ) {
			return true;
		}
		
		return false;
	}
}